;Betreibt den Signalteil des Commander 2002 Fernsteuersenders
;Gemessen werden die 5 Steuerkommandos der Kanle ber A/-Wandler
;Ausgegeben wird sowohl das Signal fr den Sendeteil, wie auch die 5 Kanle parallel
;;--------------------------------------------------------------------------
;; Connections
;;  AN0-Pin2: Eingang Steuerknppel Kanal 1
;;  AN1-Pin3: Eingang Steuerknppel Kanal 2
;;  AN2-Pin4: Eingang Steuerknppel Kanal 3
;;  AN3-Pin5: Eingang Steuerknppel Kanal 4
;;  AN4-Pin7: Eingang Steuerknppel Kanal 5
;;  RB5-Pin26: Ausgang Steuersignal an Sendeteil (invertiert, wegen nachfolgender Transistorinvertierung)
;;  RC1-Pin12: Ausgang Pulssignal Kanal 1
;;  RC2-Pin13: Ausgang Pulssignal Kanal 2
;;  RC3-Pin14: Ausgang Pulssignal Kanal 3
;;  RC4-Pin15: Ausgang Pulssignal Kanal 4
;;  RC5-Pin16: Ausgang Pulssignal Kanal 5
;;--------------------------------------------------------------------------
;;
; Je nach Einstellung des entsprechenden Kanalpotis wir ein Kanalsignal erzeugt. Dieses ist zwischen 1ms und 2ms lang.
; - Das Kanalsignal an den Sender ist ein Puls von 0,5ms Lnge gefolgt von einer Pause mit der Lnge des restlichen Kanalsiganls (also zw. 0,5ms und 1,5ms lang).
; Danach folgt das Kanalsignal des nchsten Kanals. Nach dem letzten Kanal folgt ein erneuter 0,5ms Puls und dann eine Synchronisier-Pause
; mit einer Lnge, so dass der gesamte Frame ca. 20ms dauert
; - Die einzelnen Parallelsignale sind Pulse mit der Lnge des jeweiligen Kanalsignals, gefolgt von der Synchronisierpause
;
;; Takt ist 4MHz.
;;
;;Config-register sollte bei Programmierung auf 11 1111 0011 0001 (3F31) stehen
;;(cp=11: code protection off, debug=1: disabled, wrt=1: writable, cpd=1: cp off, lvp=0: GIO enabled, 
;;boren=0: bor disabled, pwrten=0: power up timer on, wdten=0: kein watchdog timer, fosc=01: externer XT oszillator)


	processor  16F870
	radix  DEC
	__CONFIG 0x3F31

;Register Adressen
INDF        EQU   0x00
TMR0        EQU   0x01
STATUS      EQU   0x03
FSR         EQU   0x04
PORTA       EQU   0x05
PORTB       EQU   0x06
PORTC       EQU   0x07
INTCON      EQU   0x0B
PIR1	    EQU   0x0C
TMR1L       EQU   0x0E
TMR1H       EQU   0x0F
T1CON       EQU   0x10
TMR2        EQU   0x11
T2CON       EQU   0x12
ADRESH      EQU   0x1E
ADCON0      EQU   0x1F
OPTIONR     EQU   0x81
TRISA       EQU   0x85
TRISB       EQU   0x86
TRISC       EQU   0x87
PIE1        EQU   0x8C
PIE2	    EQU   0x8D
PR2         EQU   0x92
ADRESL      EQU   0x9E
ADCON1      EQU   0x9F
EEDATA	    EQU   0x10C
EEADR	    EQU   0x10D
EECON1	    EQU	  0x18C
EECON2      EQU   0x18D

_C      	EQU 	0		;Carry-Bit
_Z      	EQU   	2		;Zero-Bit
_W			EQU		0		;Destination W bei z.B. INCF
_R			EQU		1		;Destination selbes Register bei z.B. INCF 

#define		_K1		PORTC,1			;Pulsausgang Kanal 1
#define		_K2 	PORTC,2			;Pulsausgang Kanal 2
#define		_K3		PORTC,3			;Pulsausgang Kanal 3
#define		_K4		PORTC,4			;Pulsausgang Kanal 4
#define		_K5		PORTC,5			;Pulsausgang Kanal 5
#define		_SetPls	BCF	PORTB,5		;Puls setzen im Signalausgang an Sendeteil
#define		_ClrPls	BSF	PORTB,5		;Puls clearen im Signalausgang an Sendeteil
#define		_BANK0	BCF STATUS,5	;Umschalten auf Bank0
#define		_BANK1	BSF STATUS,5	;Umschalten auf Bank1
#define		_ADGO	ADCON0,2		;Umschalten auf Bank1
#define		_T2TK	PIR1,1			;Timer2 Overflow Flag

;Memory
MEM         EQU   0x20
MWert      	EQU   MEM+0		;Messwert des ADC
Paus1	    EQU   MEM+1		;Errechnete Pausendauer des Kanal 1
Paus2	    EQU   MEM+2		;Errechnete Pausendauer des Kanal 2
Paus3	    EQU   MEM+3		;Errechnete Pausendauer des Kanal 3
Paus4	    EQU   MEM+4		;Errechnete Pausendauer des Kanal 4
Paus5	    EQU   MEM+5		;Errechnete Pausendauer des Kanal 5	
a           EQU   MEM+6		;Allgemeiner Zaehler
ErgL		EQU	  MEM+7		;Ergebnis Lowbyte
ErgH		EQU	  MEM+8		;Ergebnis Highbyte
ParL		EQU	  MEM+9		;Parameter Lowbyte
ParH		EQU	  MEM+10	;Parameter Midbyte
PA			EQU	  MEM+11	;A Parameter (ADC-Min-Wert)

;Konstanten
_A1			EQU		.95		;Minimaler ADC-Wert des Kanal 1
_A2			EQU		.102	;Minimaler ADC-Wert des Kanal 2
_A3			EQU		.0		;Minimaler ADC-Wert des Kanal 3
_A4			EQU		.0		;Minimaler ADC-Wert des Kanal 4
_A5			EQU		.0		;Minimaler ADC-Wert des Kanal 5
_B1			EQU		.160	;Maximaler ADC-Wert des Kanal 1
_B2			EQU		.160	;Maximaler ADC-Wert des Kanal 2
_B3			EQU		.255	;Maximaler ADC-Wert des Kanal 3
_B4			EQU		.255	;Maximaler ADC-Wert des Kanal 4
_B5			EQU		.255	;Maximaler ADC-Wert des Kanal 5
;Das folgende sind die Skalierparameter (250*256/(b-a)), jeweils Low- und Highbyte
_P1L		EQU		.217	;Kanal 1: a=95, b=160 -> Ergebnis=985
_P1H		EQU		.3
_P2L		EQU		.79		;Kanal 2: a=102, b=160 -> Ergebnis=1103
_P2H		EQU		.4
_P3L		EQU		.251	;Kanal 3: a=0, b=255 -> Ergebnis=251
_P3H		EQU		.0
_P4L		EQU		.251	;Kanal 4: a=0, b=255 -> Ergebnis=251
_P4H		EQU		.0
_P5L		EQU		.251	;Kanal 5: a=0, b=255 -> Ergebnis=251
_P5H		EQU		.0




;******* Programm Start **************************************************
		ORG		0
		GOTO    main	


;******* DelayX *******************************************************
;einstellbares delay. Wartet 492us + W*4us
;d.h. bei W=0: 492us // bei W=250: 1492us
delayx	MOVWF	a
		BTFSC	STATUS,_Z	;Ist a=0?
		GOTO	dly2		;Ja: Diese Waitloop berspringen
		NOP	
		NOP					;W*4us warten
		DECFSZ	a,_R
		GOTO	$-2	
		NOP
		;Bis hierhin wurde jetzt mit dem aufrufenden CALL W*4+6us gewartet
		;D.h. im folgenden werten noch 486us gewartet	
dly2	MOVLW 	.120		
		MOVWF 	a	
		NOP					
		DECFSZ	a,_R
		GOTO	$-2	
		NOP
		NOP
		NOP
		RETURN

;******* DoPuls *******************************************************
;erzeugt den Fixpuls von 0,5ms Lnge
;mit aufrufendem CALL sieht das Signal folgendermaen aus:
;3us Pause - 500us Puls - 2us Pause // d.h. insgesamt 505us 
DoPuls	_SetPls				;Puls setzen
		MOVLW	.124
		MOVWF	a
		NOP
		DECFSZ	a,_R
		GOTO	$-2
		NOP
		NOP
		_ClrPls				;Puls clearen
		RETURN


;******* KCalc *******************************************************
;rechnet den Messwert vom ADC (a...b) proportional in einen Pausenwerte fr delayx (0...250) um
;Berechnung: Erg=(MWert-a)*250/(b-a)
;skaliert mit 256: Erg=(MWert-a)*(250*256/(b-a))/256
;b wird in W bergeben, a liegt in PA, (250*256/(b-a)) liegt in ParL und ParH
KCalc	SUBWF	MWert,_W	;W=Mwert-B
		BTFSC	STATUS,_C	;Borrow in Subtraktion (MWert<B)?
		RETLW	.250		;Nein: Maximalwert zurckgeben
		MOVF	PA,_W
		SUBWF	MWert,_R	;MWert=MWert-A
		BTFSS	STATUS,_C	;Borrow in Subtraktion (MWert<A)?
		RETLW	.0			;Ja: Minimalwert zurckgeben
		CALL	M212v		;Erg=Mwert*Par
		MOVF	ErgH,_W		;Highbyte von Erg nach W -> W=Erg/256
		RETURN


;;******* M212v *******************************************************
;Multiplikation 16Bit * 8Bit = 16Bit
;Funktion: Erg=MWert*Par(2)
M212v	CLRF	ErgL		;Clear Lowbyte des Ergebnisses
		CLRF	ErgH		;Clear Highbyte des Ergebnisses
		MOVLW   .8			;8-fach Schleife
		MOVWF   a
Mloop	RRF		MWert,1		;Right shift MWert nach carry
		BTFSC	STATUS, 0	;Check geshiftetes Bit
		CALL	A222v		;Wenn 1, Erg=Erg+Par
		CALL	SL2v		;Left shift Par	
		DECFSZ	a,1			;Schleife runterzhlen
		GOTO	Mloop		;Neue Schleife
		RETURN


;;******* A222v *******************************************************
;Addition 16Bit + 16Bit = 16Bit
;Funktion: Erg=Erg+Par 
A222v	MOVF	ErgH,0
		ADDWF	ParH,0		
		MOVWF	ErgH		;ErgH=ErgH+ParH
		MOVF	ErgL,0
		ADDWF	ParL,0
		MOVWF	ErgL		;ErgL=ErgL+ParL
		BTFSC	STATUS,0	;Pruefe, ob Ueberlauf
		INCF	ErgH,1		;berlauf: Erhoehe ErgH
		RETURN

;;******* SL2v *******************************************************
;LeftShift 16Bit -> 16Bit
;Funktion: Leftshift von Par (16 Bit) 
SL2v	BCF		STATUS,_C	;mit 0 auffllen	
		RLF		ParL,1		;Left shift lowbyte, oberstes Bit geht nach Carry
		RLF		ParH,1		;Left shift highbyte, unterstes Bit kommt von Carry
		RETURN



;***********Main ********************************************
main	CLRF 	INTCON
		CLRF  	PORTA
		MOVLW	0x20		;PORTB: Bit 5 = Signalausgang=1. Durch externe Transistorinvertierung wird das zu 0
		MOVWF 	PORTB
		CLRF 	PORTC
		CLRF 	T1CON		;T1 aus	
		MOVLW 	0x27		;T2 init: Postcale=1:5, Prescale=1:16
		MOVWF 	T2CON
		CLRF  	TMR2
		MOVLW 	0x41		;ADC initialisieren
		MOVWF 	ADCON0
		_BANK1 				;Bank 1
		MOVLW	.249		;PR2 Init: Pr2=249
		MOVWF	PR2			;Damit flagt TMR2 alle 16*250*5=20000us=20ms
		MOVLW 	0x02
		MOVWF 	ADCON1
		CLRF 	PIE1		;PIE1 Init
		CLRF  	PIE2
		MOVLW 	0x87		;Options			
		MOVWF 	OPTIONR			
		MOVLW 	0xFF		;PortA - Alles Inputs: 1111 1111
		MOVWF 	TRISA
		MOVLW 	0xDF		;PortB - Bit 5 ist Output, sonst alles Input: 1101 1111
		MOVWF  	TRISB
		MOVLW 	0xC1		;PortC - 1,2,3,4,5 sind output, Rest Input: 1100 0001
		MOVWF 	TRISC
		_BANK0 				;Bank 0


	;********* Kanalmessungen und Umrechnung ******************
Mess	;Kanal 1 messen und umrechnen
		BSF   	_ADGO		;Starte ADC Conversion auf AN0 (ADC ist schon auf RC-Kanal 1 (=AN0)
		BTFSC 	_ADGO		;Auf Ende der Konvertierung warten (GO=0)
		GOTO  	$-1	
		MOVF  	ADRESH,_W	;ADRESH nach MWert speichern	
		MOVWF 	MWert
		MOVLW 	0x49		;ADC auf RC-Kanal 2 (=AN1) schalten, damit S&H whrend folgender Umrechnung aufldt
		MOVWF 	ADCON0
		MOVLW	_P1L		;Berechnungsparameter festlegen
		MOVWF	ParL
		MOVLW	_P1H
		MOVWF	ParH
		MOVLW	_A1
		MOVWF	PA	
		MOVLW	_B1	
		CALL	KCalc		;RC-Kanal 1 Messwert umrechnen (dient auch als acquisition time fr nchste Messung) und...
		MOVWF	Paus1		;nach Paus1 schreiben
		;Kanal 2 messen und umrechnen
		BSF   	_ADGO		;Starte ADC Conversion auf AN1
		BTFSC 	_ADGO		;Auf Ende der Konvertierung warten (GO=0)
		GOTO  	$-1	
		MOVF  	ADRESH,_W	;ADRESH nach MWert speichern	
		MOVWF 	MWert
		MOVLW 	0x51		;ADC auf RC-Kanal 3 (=AN2) schalten, damit S&H whrend folgender Umrechnung aufldt
		MOVWF 	ADCON0
		MOVLW	_P2L		;Berechnungsparameter festlegen
		MOVWF	ParL
		MOVLW	_P2H
		MOVWF	ParH
		MOVLW	_A2
		MOVWF	PA		
		MOVLW	_B2			
		CALL	KCalc		;RC-Kanal 2 Messwert umrechnen (dient auch als acquisition time fr nchste Messung) und...
		MOVWF	Paus2		;nach Paus2 schreiben
		;Kanal 3 messen und umrechnen
		BSF   	_ADGO		;Starte ADC Conversion auf AN2
		BTFSC 	_ADGO		;Auf Ende der Konvertierung warten (GO=0)
		GOTO  	$-1	
		MOVF  	ADRESH,_W	;ADRESH nach MWert speichern	
		MOVWF 	MWert
		MOVLW 	0x59		;ADC auf RC-Kanal 4 (=AN3) schalten, damit S&H whrend folgender Umrechnung aufldt
		MOVWF 	ADCON0	
		MOVLW	_P3L		;Berechnungsparameter festlegen
		MOVWF	ParL
		MOVLW	_P3H
		MOVWF	ParH
		MOVLW	_A3
		MOVWF	PA		
		MOVLW	_B3		
		CALL	KCalc		;RC-Kanal 3 Messwert umrechnen (dient auch als acquisition time fr nchste Messung) und...
		MOVWF	Paus3		;nach Paus3 schreiben
		;Kanal 4 messen und umrechnen
		BSF   	_ADGO		;Starte ADC Conversion auf AN3
		BTFSC 	_ADGO		;Auf Ende der Konvertierung warten (GO=0)
		GOTO  	$-1	
		MOVF  	ADRESH,_W	;ADRESH nach MWert speichern	
		MOVWF 	MWert
		MOVLW 	0x61		;ADC auf RC-Kanal 5 (=AN4) schalten, damit S&H whrend folgender Umrechnung aufldt
		MOVWF 	ADCON0	
		MOVLW	_P4L		;Berechnungsparameter festlegen
		MOVWF	ParL
		MOVLW	_P4H
		MOVWF	ParH
		MOVLW	_A4
		MOVWF	PA		
		MOVLW	_B4		
		CALL	KCalc		;RC-Kanal 4 Messwert umrechnen (dient auch als acquisition time fr nchste Messung) und...
		MOVWF	Paus4		;nach Paus4 schreiben
		;Kanal 5 messen und umrechnen
		BSF   	_ADGO		;Starte ADC Conversion auf AN4
		BTFSC 	_ADGO		;Auf Ende der Konvertierung warten (GO=0)
		GOTO  	$-1	
		MOVF  	ADRESH,_W	;ADRESH nach MWert speichern	
		MOVWF 	MWert
		MOVLW 	0x41		;ADC auf RC-Kanal 1 (=AN0) schalten, damit S&H whrend folgender Umrechnung aufldt
		MOVWF 	ADCON0
		MOVLW	_P5L		;Berechnungsparameter festlegen
		MOVWF	ParL
		MOVLW	_P5H
		MOVWF	ParH
		MOVLW	_A5
		MOVWF	PA		
		MOVLW	_B5			
		CALL	KCalc		;RC-Kanal 5 Messwert umrechnen (dient auch als acquisition time fr nchste Messung) und...
		MOVWF	Paus5		;nach Paus5 schreiben

		;Warten bis Frame-Zeit (20ms) um ist (=TMR2-berlauf)
		BTFSS	_T2TK		;Warten bis Timer2-Flag aktiviert (nach 20ms)
		GOTO	$-1
		BCF		_T2TK		;Flag clearen

		;Ab hier Ausgabe der Kanalsignal, sowohl ber die Signal-Line wie auch an die einzelnen Parallelsignale
		;Kanal 1
		BSF		_K1			;Parallelsignal setzen
		CALL	DoPuls		;Puls aus Signal-Line
		MOVF	Paus1,_W	;Pausen zeit von Kanal 1...
		CALL	delayx		;...warten
		MOVLW	0x04
		;Kanal 2
		MOVWF	PORTC		;K1 clearen, K2 setzen
		CALL	DoPuls		;Puls aus Signal-Line
		MOVF	Paus2,_W	;Pausen zeit von Kanal 2...
		CALL	delayx		;...warten
		MOVLW	0x08
		;Kanal 3
		MOVWF	PORTC		;K2 clearen, K3 setzen
		CALL	DoPuls		;Puls aus Signal-Line
		MOVF	Paus3,_W	;Pausen zeit von Kanal 3...
		CALL	delayx		;...warten
		MOVLW	0x10
		;Kanal 4
		MOVWF	PORTC		;K3 clearen, K4 setzen
		CALL	DoPuls		;Puls aus Signal-Line
		MOVF	Paus4,_W	;Pausen zeit von Kanal 4...
		CALL	delayx		;...warten
		MOVLW	0x20
		;Kanal 5
		MOVWF	PORTC		;K4 clearen, K5 setzen
		CALL	DoPuls		;Puls aus Signal-Line
		MOVF	Paus5,_W	;Pausen zeit von Kanal 5...
		CALL	delayx		;...warten
		NOP					;Als Ersatz fr MOVLW
		BCF		_K5			;K5 clearen

		;Frame beenden
		CALL	DoPuls		;Abschluss-Puls setzen
		GOTO	Mess		;Start von vorne, Messungen finden whrend der Sync-Pause statt

		END

